@@ -17,16 +17,16 @@ Questions, bugs and patches should be sent to instant@lists.launchpad.net.
 """
 
 __authors__ = "Magne Westlie, Kent-Andre Mardal <kent-and@simula.no>, Martin Alnes <martinal@simula.no>, Ilmar M. Wilbers <ilmarw@simula.no>"
-__date__ = "2011-12-07"
-__version__ = "1.0.0"
+__date__ = "2013-03-24"
+__version__ = "1.2.0"
 
 # TODO: Import only the official interface
-from output import *
-from config import *
-from paths import *
-from signatures import *
-from cache import *
-from codegeneration import *
-from build import *
-from inlining import *
+from .output import *
+from .config import *
+from .paths import *
+from .signatures import *
+from .cache import *
+from .codegeneration import *
+from .build import *
+from .inlining import *
 
