@@ -156,36 +156,36 @@ def write_interfacefile(filename, modulename, code, init_code,
     numpy_i_include = ''
     if arrays:
         numpy_i_include = r'%include "numpy.i"'
-    
-    interface_string = reindent("""
-        %%module  %(modulename)s
-        //%%module (directors="1") %(modulename)s
 
-        //%%feature("director");
+    # Do not reindent as SWIG interface code can also include Python code.
+    interface_string = """%%module  %(modulename)s
+//%%module (directors="1") %(modulename)s
 
-        %%{
-        #include <iostream>
-        %(additional_definitions)s 
-        %(system_headers_code)s 
-        %(local_headers_code)s 
-        %(wrap_headers_code1)s 
-        %(code)s
-        %%}
+//%%feature("director");
 
-        //%%feature("autodoc", "1");
-        %(numpy_i_include)s
-        
-        %%init%%{
-        %(init_code)s
-        %%}
+%%{
+#include <iostream>
+%(additional_definitions)s 
+%(system_headers_code)s 
+%(local_headers_code)s 
+%(wrap_headers_code1)s 
+%(code)s
+%%}
 
-        %(additional_definitions)s
-        %(additional_declarations)s
-        %(wrap_headers_code2)s
-        //%(typemaps)s
-        %(code)s;
+//%%feature("autodoc", "1");
+%(numpy_i_include)s
 
-        """ % locals())
+%%init%%{
+%(init_code)s
+%%}
+
+%(additional_definitions)s
+%(additional_declarations)s
+%(wrap_headers_code2)s
+//%(typemaps)s
+%(code)s;
+
+""" % locals()
     
     write_file(filename, interface_string)
     instant_debug("Done generating interface file.")
@@ -344,8 +344,95 @@ def generate_interface_file_vtk(signature, code):
     s = interface_template % { "typemaps" : typemaps, "code" : code, "includes" : includes } 
     return s
 
+def write_cmakefile(module_name, cmake_packages):
+    find_package_template = """
+# Configuration for package %(package)s
+FIND_PACKAGE(%(package)s REQUIRED)
+IF(%(package)s_FOUND)
+ INCLUDE(${%(PACKAGE)s_USE_FILE})
+ENDIF(%(package)s_FOUND)
 
-def write_cmakefile(name):    
+if (NOT $ENV{CXX})
+  set(CMAKE_CXX_COMPILER ${%(PACKAGE)s_CXX_COMPILER})
+endif()
+"""    
+    cmake_form = dict(module_name=module_name)
+    cmake_form["find_packages"] = "\n\n".join(find_package_template % \
+                                              dict(package=package,
+                                                   PACKAGE=package.upper())\
+                                              for package in cmake_packages)
+    cmake_form["packages_definitions"] = "\n".join(
+        "${%s_CXX_DEFINITIONS}" % package.upper()
+        for package in cmake_packages)
+
+    cmake_form["package_include_dirs"] = "\n".join(\
+        "include_directories(${%s_PYTHON_INCLUDE_DIRS} ${${NAME}_SOURCE_DIR})" %
+        package.upper() for package in cmake_packages)
+
+    cmake_form["package_flags"] = "\n".join(\
+        """set(CMAKE_CXX_FLAGS \"${CMAKE_CXX_FLAGS} ${%(package)s_CXX_FLAGS}\")
+set(CMAKE_EXE_LINKER_FLAGS \"${CMAKE_EXE_LINKER_FLAGS} ${%(package)s_LINK_FLAGS}\")
+set(CMAKE_SHARED_LINKER_FLAGS \"${CMAKE_SHARED_LINKER_FLAGS} ${%(package)s_LINK_FLAGS}\")
+""" %
+        dict(package=package.upper()) for package in cmake_packages)
+
+    cmake_form["package_swig_link_libraries"] = "\n".join(\
+        """if (DEFINED %(package)s_LIBRARIES OR DEFINED %(package)s_3RD_PARTY_LIBRARIES OR DEFINED %(package)s_PYTHON_LIBRARIES)
+  swig_link_libraries(${SWIG_MODULE_NAME} ${%(package)s_LIBRARIES} ${%(package)s_3RD_PARTY_LIBRARIES} ${%(package)s_PYTHON_LIBRARIES})
+endif()""" %
+        dict(package=package.upper()) for package in cmake_packages)
+    
+    cmake_template = """
+cmake_minimum_required(VERSION 2.6.0)
+
+# This project is designed to be built outside the Insight source tree.
+set (NAME %(module_name)s)
+
+%(find_packages)s
+
+PROJECT(${NAME})
+
+find_package(SWIG REQUIRED)
+include(${SWIG_USE_FILE})
+
+set(SWIG_MODULE_NAME ${NAME})
+set(CMAKE_SWIG_FLAGS
+  -module ${SWIG_MODULE_NAME}
+  -shadow
+  -modern
+  -modernargs
+  -fastdispatch
+  -fvirtual
+  -nosafecstrings
+  -noproxydel
+  -fastproxy
+  -fastinit
+  -fastunpack
+  -fastquery
+  -nobuildnone
+%(packages_definitions)s
+  )
+
+set(CMAKE_SWIG_OUTDIR ${CMAKE_CURRENT_BINARY_DIR})
+
+set(SWIG_SOURCES ${NAME}.i)
+
+set_source_files_properties(${SWIG_SOURCES} PROPERTIES CPLUSPLUS ON)
+
+%(package_include_dirs)s
+
+%(package_flags)s
+
+swig_add_module(${SWIG_MODULE_NAME} python ${SWIG_SOURCES})
+
+%(package_swig_link_libraries)s
+
+""" % cmake_form
+
+    filename = "CMakeLists.txt"
+    write_file(filename, cmake_template)
+
+def write_itk_cmakefile(name):    
     file_template = """
 cmake_minimum_required(VERSION 2.6.0)
 
