@@ -24,11 +24,11 @@ a = numpy.arange(10000000); a = numpy.sin(a)
 t1 = time.time()
 b = some_func(a)
 t2 = time.time()
-print 'With instant:',t2-t1,'seconds'
+print('With instant:',t2-t1,'seconds')
 
 t1 = time.time()
 c = sum(numpy.sin(a) + numpy.cos(a))
 t2 = time.time()
-print 'With  numpy :',t2-t1,'seconds'
+print('With  numpy :',t2-t1,'seconds')
 
-print 'Difference: ', c - b  
+print('Difference: ', c - b)  
